// hlavickove subory *******************************************************
#include "includes.h"
// vlastny .h subor
#include "interface.h"

// uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
GLfloat dTime;

// pre pracu s matematikou
Cmath3d mm;

// instancia triedy CTexture - praca s texturami
CTexture texture;
// ciselne oznacenia textur - nie su totozne s identifikatormi nahratych textur v CTexture!
// vyhradene oznacenia:
// 0 - prazdna (empty) textura
// 1 - standardna (default) textura
// 2 - zameriavaci kriz
// <10;(MAX_TEX_NUM-1)> - ostatne textury
GLuint tid[MAX_TEX_NUM];

// instancia triedy CText - praca s textom
CText text;

// pocet segmentov gule, valca, ihlana
int segs=18;
// pocet udajov / 3 = pocet trojuholnikov
/*
int data_num=108+                          // kocka
             (360/segs*18)*(180/segs)+     // gula
             (360/segs*18)+(360/segs*9)*2+ // valec
             (360/segs*9)+(360/segs*9)+    // ihlan
             18+18;                        // plochy
*/
// vrcholy, farba, TBN, texturovacie suradnice
// 108 + 3600 + 720 + 360 + 18 + 18 = 4824
float v_a[4824],c_a[4824],tan_a[4824],bit_a[4824],nor_a[4824],t_a[4824];

// vektory totozne so suradnicovymi osami
vec3 rightV(1.0f,0.0f,0.0f);
vec3 upV(0.0f,1.0f,0.0f);
vec3 forwardV(0.0f,0.0f,-1.0f);

// konstruktor *************************************************************
CInterface::CInterface()
{

}

// destruktor **************************************************************
CInterface::~CInterface()
{
 // Vertex Array Object
 //glDeleteVertexArrays(1,&vao);
}

// inicializacia OpenGL - nastavenie parametrov OpenGL sceny ***************
int CInterface::OGL_initialize(void)
{
 printf("OpenGL init...\n");

 GLenum err=glewInit();
 if (err!=GLEW_OK) { printf("GLEW error: %s\n",glewGetErrorString(err)); return(1); }
 printf("using GLEW %s\n",glewGetString(GLEW_VERSION));

 // informacia o verzii OpenGL
 int glmaj,glmin;
 getglver(&glmaj,&glmin);
 printf("using OpenGL %d.%d\n",glmaj,glmin);
 if (glmaj<2) {
  printf("Pre spustenie tejto aplikacie je potrebna kniznica OpenGL 2.0\n");
  return(1);
 }
 printf("using GLSL %s\n",getglslver());
 val=glGetString(GL_VENDOR); printf("vendor: %s ",val);
 val=glGetString(GL_RENDERER); printf("renderer: %s\n",val);

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // farba, ktorou sa bude resetovat zasobnik farieb
 glClearColor(0.2f,0.2f,0.2f,1.0f);
 // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 glClearDepth(1.0f);
 // pomocny zasobnik bude vymazavany nastavenim na 0.0f
 glClearStencil(0.0f);

 glDepthFunc(GL_LESS);    // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // testovanie Z-vzdialenosti

 glDisable(GL_CULL_FACE); // poradie, v ktorom budu vrcholy objektov
 glFrontFace(GL_CCW);     // definovat privratene, resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 //glPolygonMode(GL_FRONT,GL_LINE);

 // osvetlenie
 lsrcparam.ambient.set(0.6f,0.6f,0.6f,1.0f);
 lsrcparam.diffuse.set(0.8f,0.8f,0.8f,1.0f);
 lsrcparam.specular.set(0.2f,0.2f,0.2f,1.0f);
 lsrcparam.position.set(4.0f,5.0f,4.0f,1.0f);
 lsrcparam.spotDirection.set(0.0f,0.0f,-1.0f);
 lsrcparam.spotExponent=20.0f;
 lsrcparam.spotCutoff=30.0f;
 lsrcparam.spotCosCutoff=cos(DEG2RAD(lsrcparam.spotCutoff));
 lsrcparam.spotSinCutoff=sin(DEG2RAD(lsrcparam.spotCutoff));
 lsrcparam.constantAttenuation=1.0f;
 lsrcparam.linearAttenuation=0.0f;
 lsrcparam.quadraticAttenuation=0.0f;

 lmodelparam.ambient.set(0.4f,0.4f,0.4f,1.0f);

 mparam.ambient.set(0.6f,0.6f,0.6f,1.0f);
 mparam.diffuse.set(0.8f,0.8f,0.8f,1.0f);
 mparam.specular.set(1.0f,1.0f,1.0f,1.0f);
 mparam.emission.set(0.0f,0.0f,0.0f,1.0f);
 mparam.shininess=128.0f;

 // aktualizacia parametrov osvetlenia
 update_lighting_params();

 // texturovanie je standardne zapnute
 glEnable(GL_TEXTURE_2D);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // nastavenie ciselnych oznaceni textur
 // vyhradene oznacenia:
 // 0 - prazdna (empty) textura
 // 1 - standardna (default) textura
 // 2 - zameriavaci kriz
 // <10;(MAX_TEX_NUM-1)> - ostatne textury
 loopi(MAX_TEX_NUM) tid[i]=i;

 // inicializacia Avatara
 avatar.init();

 // inicializacia textu - nahratie textury s fontom
 if (text.init("data/font/font.png")) {
  printf("Chyba inicializacie textu!\n"); return(1);
 }

 // nahratie prazdnej (empty) textury
 if (texture.load2D(tid[0],"data/textures/default/empty.png",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia prazdnej textury!\n"); return(1);
 }
 // nahratie standardnej (default) textury
 if (texture.load2D(tid[1],"data/textures/default/default.png",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia standardnej textury!\n"); return(1);
 }
 // nahratie textury zameriavacieho kriza
 if (texture.load2D(tid[2],"data/gfx/crosshair1.png",T_LINEAR | T_CLAMP)) {
  printf("Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 // priprava 3D textury
 if (texture.prepare_perlin_noise_tex(tid[3])) {
  printf("Chyba pripravy 3D textury!\n"); return(1);
 }
 // nahratie textury predstavujucej svetelny zdroj
 if (texture.load2D(tid[10],"data/textures/lamp.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury svetelneho zdroja!\n"); return(1);
 }

 // nahratie textury
 if (texture.load2D(tid[11],"data/textures/chessboard.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }
 // nahratie normalmap textury
 if (texture.load2D(tid[12],"data/textures/chessboard_nm.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }

 // nahratie textury
 if (texture.load2D(tid[13],"data/textures/px3/color_map.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }
 // nahratie heightmap textury
 if (texture.load2D(tid[14],"data/textures/px3/height_map.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }
 // nahratie normalmap textury
 if (texture.load2D(tid[15],"data/textures/px3/normal_map.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }

 // nahratie textury
 if (texture.load2D(tid[16],"data/textures/ceramic.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia textury!\n"); return(1);
 }

 // velkost zameriavacieho kriza
 crosshairSize=50.0f;

 // tlacidla menu
 but[0].init(vec3(10.0f,75.0f,0.0f),30.0f,15.0f); but[1].init(vec3(45.0f,75.0f,0.0f),30.0f,15.0f);
 but[2].init(vec3(80.0f,75.0f,0.0f),30.0f,15.0f); but[3].init(vec3(115.0f,75.0f,0.0f),30.0f,15.0f);

 but[4].init(vec3(10.0f,105.0f,0.0f),30.0f,15.0f); but[5].init(vec3(45.0f,105.0f,0.0f),30.0f,15.0f);
 but[6].init(vec3(80.0f,105.0f,0.0f),30.0f,15.0f); but[7].init(vec3(115.0f,105.0f,0.0f),30.0f,15.0f);

 but[8].init(vec3(10.0f,135.0f,0.0f),30.0f,15.0f); but[9].init(vec3(45.0f,135.0f,0.0f),30.0f,15.0f);
 but[10].init(vec3(80.0f,135.0f,0.0f),30.0f,15.0f); but[11].init(vec3(115.0f,135.0f,0.0f),30.0f,15.0f);

 but[12].init(vec3(10.0f,165.0f,0.0f),30.0f,15.0f); but[13].init(vec3(45.0f,165.0f,0.0f),30.0f,15.0f);
 but[14].init(vec3(80.0f,165.0f,0.0f),30.0f,15.0f); but[15].init(vec3(115.0f,165.0f,0.0f),30.0f,15.0f);

 but[16].init(vec3(10.0f,195.0f,0.0f),30.0f,15.0f); but[17].init(vec3(45.0f,195.0f,0.0f),30.0f,15.0f);
 but[18].init(vec3(80.0f,195.0f,0.0f),30.0f,15.0f);

 but[19].init(vec3(10.0f,225.0f,0.0f),36.0f,15.0f);
 but[20].init(vec3(10.0f,255.0f,0.0f),36.0f,15.0f);

 but[21].init(vec3(10.0f,285.0f,0.0f),30.0f,15.0f);
 but[22].init(vec3(10.0f,315.0f,0.0f),30.0f,15.0f);
 but[23].init(vec3(10.0f,345.0f,0.0f),30.0f,15.0f);

 but[24].init(vec3(10.0f,375.0f,0.0f),30.0f,15.0f); but[25].init(vec3(45.0f,375.0f,0.0f),30.0f,15.0f);
 but[26].init(vec3(80.0f,375.0f,0.0f),30.0f,15.0f); but[27].init(vec3(115.0f,375.0f,0.0f),30.0f,15.0f);

 but[28].init(vec3(10.0f,425.0f,0.0f),30.0f,15.0f); but[29].init(vec3(45.0f,425.0f,0.0f),30.0f,15.0f);
 but[30].init(vec3(80.0f,425.0f,0.0f),30.0f,15.0f); but[31].init(vec3(115.0f,425.0f,0.0f),30.0f,15.0f);

 but[32].init(vec3(10.0f,455.0f,0.0f),30.0f,15.0f); but[33].init(vec3(45.0f,455.0f,0.0f),30.0f,15.0f);
 but[34].init(vec3(80.0f,455.0f,0.0f),30.0f,15.0f); but[35].init(vec3(115.0f,455.0f,0.0f),30.0f,15.0f);

 but[36].init(vec3(10.0f,485.0f,0.0f),30.0f,15.0f); but[37].init(vec3(45.0f,485.0f,0.0f),30.0f,15.0f);
 but[38].init(vec3(80.0f,485.0f,0.0f),30.0f,15.0f); but[39].init(vec3(115.0f,485.0f,0.0f),30.0f,15.0f);

 but[40].init(vec3(10.0f,515.0f,0.0f),30.0f,15.0f); but[41].init(vec3(45.0f,515.0f,0.0f),30.0f,15.0f);
 but[42].init(vec3(80.0f,515.0f,0.0f),30.0f,15.0f); but[43].init(vec3(115.0f,515.0f,0.0f),30.0f,15.0f);

 but[44].init(vec3(10.0f,545.0f,0.0f),36.0f,15.0f);

 but[45].init(vec3(10.0f,575.0f,0.0f),135.0f,15.0f);

 but[46].init(vec3(95.0f,560.0f,0.0f),50.0f,14.0f);

 loopi(BUTNUM) but[i].set_tscale(0.3f,0.3f); // velkost textu tlacidiel

 // parametre a udaje tlacidiel
 but[0].set_data(lsrcparam.ambient.x); but[1].set_data(lsrcparam.ambient.y);
 but[2].set_data(lsrcparam.ambient.z); but[3].set_data(lsrcparam.ambient.w);

 but[4].set_data(lsrcparam.diffuse.x); but[5].set_data(lsrcparam.diffuse.y);
 but[6].set_data(lsrcparam.diffuse.z); but[7].set_data(lsrcparam.diffuse.w);

 but[8].set_data(lsrcparam.specular.x); but[9].set_data(lsrcparam.specular.y);
 but[10].set_data(lsrcparam.specular.z); but[11].set_data(lsrcparam.specular.w);

 but[12].set_range(-100.0f,100.0f); but[13].set_range(-100.0f,100.0f);
 but[14].set_range(-100.0f,100.0f);
 but[12].set_data(lsrcparam.position.x); but[13].set_data(lsrcparam.position.y);
 but[14].set_data(lsrcparam.position.z); but[15].set_data(lsrcparam.position.w);

 but[16].set_range(-1.0f,1.0f); but[17].set_range(-1.0f,1.0f);
 but[18].set_range(-1.0f,1.0f);
 but[16].set_data(lsrcparam.spotDirection.x); but[17].set_data(lsrcparam.spotDirection.y);
 but[18].set_data(lsrcparam.spotDirection.z);

 but[19].set_range(0.0f,128.0f); but[19].set_cnum(5);
 but[19].set_data(lsrcparam.spotExponent);
 but[20].set_range(0.0f,90.0f); but[20].set_cnum(5);
 but[20].set_data(lsrcparam.spotCutoff);

 but[21].set_data(lsrcparam.constantAttenuation);
 but[22].set_data(lsrcparam.linearAttenuation);
 but[23].set_data(lsrcparam.quadraticAttenuation);

 but[24].set_data(lmodelparam.ambient.x); but[25].set_data(lmodelparam.ambient.y);
 but[26].set_data(lmodelparam.ambient.z); but[27].set_data(lmodelparam.ambient.w);

 but[28].set_data(mparam.ambient.x); but[29].set_data(mparam.ambient.y);
 but[30].set_data(mparam.ambient.z); but[31].set_data(mparam.ambient.w);

 but[32].set_data(mparam.diffuse.x); but[33].set_data(mparam.diffuse.y);
 but[34].set_data(mparam.diffuse.z); but[35].set_data(mparam.diffuse.w);

 but[36].set_data(mparam.specular.x); but[37].set_data(mparam.specular.y);
 but[38].set_data(mparam.specular.z); but[39].set_data(mparam.specular.w);

 but[40].set_data(mparam.emission.x); but[41].set_data(mparam.emission.y);
 but[42].set_data(mparam.emission.z); but[43].set_data(mparam.emission.w);

 but[44].set_range(0.0f,128.0f); but[44].set_cnum(5);
 but[44].set_data(mparam.shininess);

 but_text_id=SHNUM-1;         // index textoveho udaju tlacidla = pouzity shader
 strcpy(but_text[0],"base");  // SHNUM x textovy udaj tlacidla = nazvy shaderov
 strcpy(but_text[1],"light");
 strcpy(but_text[2],"brick");
 strcpy(but_text[3],"toon");
 strcpy(but_text[4],"wave");
 strcpy(but_text[5],"wood");
 strcpy(but_text[6],"normalmap");
 strcpy(but_text[7],"parallax");
 strcpy(but_text[8],"water");
 strcpy(but_text[9],"dof");
 but[45].set_range(0.0f,9.1f);                        // po konverzii na int bude ciselny udaj 0-9
 but[45].set_show_data(1);                            // tlacidlo bude zobrazovat textovy udaj
 but[45].set_data(but_text_id,but_text[but_text_id]); // ciselny a textovy udaj tlacidla

 but[46].set_show_data(1);     // tlacidlo bude zobrazovat textovy udaj
 but[46].set_data(0,"reload"); // ciselny a textovy udaj tlacidla

 but_id=-1; // index aktivneho tlacidla

 // Vertex Array Object
 //glGenVertexArrays(1,&vao);
 //glBindVertexArray(vao);

 // FBO
 fbo.init();

 // priprava objektov - kocka, gula, valec, ihlan, plochy
 objects_prepare();

 // inicializacia a nacitanie shaderov
 if (base_shader.init()) return(1);
 if (base_shader.load("shaders/base.vp","shaders/base.fp")) return(1);
 if (light_shader.load("shaders/light.vp","shaders/light.fp")) return(1);
 if (brick_shader.load("shaders/brick.vp","shaders/brick.fp")) return(1);
 if (toon_shader.load("shaders/toon.vp","shaders/toon.fp")) return(1);
 if (wave_shader.load("shaders/wave.vp","shaders/wave.fp")) return(1);
 if (wood_shader.load("shaders/wood.vp","shaders/wood.fp")) return(1);
 if (nm_shader.load("shaders/normalmap.vp","shaders/normalmap.fp")) return(1);
 if (px_shader.load("shaders/parallax.vp","shaders/parallax.fp")) return(1);
 if (water_shader.load("shaders/water.vp","shaders/water.fp")) return(1);
 if (dof_shader.init()) return(1);

 base_shader.on();
 base_shader.bindattriblocation(0,"in_pos"); // vrcholy
 base_shader.bindattriblocation(1,"in_col"); // farby
 base_shader.bindattriblocation(4,"in_nor"); // normaly
 base_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 base_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 base_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 base_shader.off();

 light_shader.on();
 light_shader.bindattriblocation(0,"in_pos"); // vrcholy
 light_shader.bindattriblocation(1,"in_col"); // farby
 light_shader.bindattriblocation(4,"in_nor"); // normaly
 light_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 light_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 light_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 light_shader.off();

 brick_shader.on();
 brick_shader.bindattriblocation(0,"in_pos"); // vrcholy
 brick_shader.bindattriblocation(4,"in_nor"); // normaly
 brick_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 brick_shader.off();

 toon_shader.on();
 toon_shader.bindattriblocation(0,"in_pos"); // vrcholy
 toon_shader.bindattriblocation(4,"in_nor"); // normaly
 toon_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 toon_shader.off();

 wave_shader.on();
 wave_shader.bindattriblocation(0,"in_pos"); // vrcholy
 wave_shader.bindattriblocation(1,"in_col"); // farby
 wave_shader.bindattriblocation(4,"in_nor"); // normaly
 wave_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 wave_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 wave_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 wave_shader.off();

 wood_shader.on();
 wood_shader.bindattriblocation(0,"in_pos"); // vrcholy
 wood_shader.bindattriblocation(4,"in_nor"); // normaly
 wood_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 wood_shader.uniform3F("lightWoodColor",0.6f,0.3f,0.1f);
 wood_shader.uniform3F("darkWoodColor",0.4f,0.2f,0.07f);
 wood_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 wood_shader.off();

 nm_shader.on();
 nm_shader.bindattriblocation(0,"in_pos"); // vrcholy
 nm_shader.bindattriblocation(2,"in_tan"); // tangenty
 nm_shader.bindattriblocation(3,"in_bit"); // bitangenty
 nm_shader.bindattriblocation(4,"in_nor"); // normaly
 nm_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 nm_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 nm_shader.uniformI("nm_tex_unit",1);      // normalmap texturovacia jednotka
 nm_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 nm_shader.off();

 px_shader.on();
 px_shader.bindattriblocation(0,"in_pos"); // vrcholy
 px_shader.bindattriblocation(2,"in_tan"); // tangenty
 px_shader.bindattriblocation(3,"in_bit"); // bitangenty
 px_shader.bindattriblocation(4,"in_nor"); // normaly
 px_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 px_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
 px_shader.uniformI("hm_tex_unit",1);      // heightmap texturovacia jednotka
 px_shader.uniformI("nm_tex_unit",2);      // normalmap texturovacia jednotka
 px_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 px_shader.off();

 // inicializacia objektu predstavujuceho hladinu vody
 if (wtr.init()) return(1);

 water_shader.on();
 water_shader.bindattriblocation(0,"in_pos"); // vrcholy
 water_shader.bindattriblocation(2,"in_tan"); // tangenty
 water_shader.bindattriblocation(3,"in_bit"); // bitangenty
 water_shader.bindattriblocation(4,"in_nor"); // normaly
 water_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
 water_shader.uniformI("rfl_tex_unit",0);     // reflection texturovacia jednotka
 water_shader.uniformI("rfr_tex_unit",1);     // refraction texturovacia jednotka
 water_shader.uniformI("depth_tex_unit",2);   // depth texturovacia jednotka
 water_shader.uniformI("nm_tex_unit",3);      // normalmap texturovacia jednotka
 water_shader.uniformI("dudv_tex_unit",4);    // dudv texturovacia jednotka
 water_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
 water_shader.off();

 // depth of field shadery - na efekte spolupracuje celkovo 5 shaderov
 dof_shader.on(0);
 dof_shader.bindattriblocation(0,0,"in_pos");
 dof_shader.bindattriblocation(0,8,"in_uv");
 dof_shader.uniformmatrixFV(0,"projection_mat",avatar.get_pr_mat());
 dof_shader.uniformI(0,"base_tex_unit",0);
 dof_shader.uniform1F(0,"focalDistance",3.0f);
 dof_shader.uniform1F(0,"focalRange",6.0f);

 dof_shader.on(1);
 dof_shader.bindattriblocation(1,0,"in_pos");
 dof_shader.bindattriblocation(1,8,"in_uv");
 dof_shader.uniformI(1,"base_tex_unit",0);

 // nastavenie poloh vzoriek Gausovho filtra
 mm.set_gaussian_filter(scr_w*2,scr_h*2);
 
 dof_shader.on(2);
 dof_shader.bindattriblocation(2,0,"in_pos");
 dof_shader.bindattriblocation(2,8,"in_uv");
 dof_shader.uniformI(2,"base_tex_unit",0);
 dof_shader.uniform4FV(2,"horzTapOffs0",mm.horzTapOffs[0].get_data());
 dof_shader.uniform4FV(2,"horzTapOffs1",mm.horzTapOffs[1].get_data());
 dof_shader.uniform4FV(2,"horzTapOffs2",mm.horzTapOffs[2].get_data());
 dof_shader.uniform4FV(2,"horzTapOffs3",mm.horzTapOffs[3].get_data());

 dof_shader.on(3);
 dof_shader.bindattriblocation(3,0,"in_pos");
 dof_shader.bindattriblocation(3,8,"in_uv");
 dof_shader.uniformI(3,"base_tex_unit",0);
 dof_shader.uniform4FV(3,"vertTapOffs0",mm.vertTapOffs[0].get_data());
 dof_shader.uniform4FV(3,"vertTapOffs1",mm.vertTapOffs[1].get_data());
 dof_shader.uniform4FV(3,"vertTapOffs2",mm.vertTapOffs[2].get_data());
 dof_shader.uniform4FV(3,"vertTapOffs3",mm.vertTapOffs[3].get_data());

 dof_shader.on(4);
 dof_shader.bindattriblocation(4,0,"in_pos");
 dof_shader.bindattriblocation(4,8,"in_uv");
 dof_shader.uniformI(4,"base_tex_unit",0);
 dof_shader.uniformI(4,"blur_tex_unit",1);

 dof_shader.off();

 lA=0.0f; lB=0.0f; // natocenie svetelneho zdroja

 return(0);
}

// nastartovanie casovaca **************************************************
// 1s=1000ms
// Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
// po 49.7 dnoch dojde k prekroceniu 32bit limitu a pocitadlo sa vynuluje
void CInterface::resettimer(void)
{
 // aktualny casovy okamih (ms)
 actTime=(GLuint) SDL_GetTicks();
 // povodny casovy okamih (ms)
 oldTime=actTime;
 // pocet uplynutych milisekund
 millis=actTime-oldTime;
 // pomocne premenne
 fpsdelay=0; fpserror=0;

 // uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
 dTime=((GLfloat) millis)/1000.0f;

 // uplynute ms
 mspos=0; mstotal=0;
 // pocet snimkov za sekundu, minimalny a maximalny pocet snimkov za sekundu
 actfps=0; minfps=0; maxfps=0;
}

// aktualny pocet fps, minimalny a maximalny pocet fps *********************
void CInterface::getfps(GLuint &ACT,GLuint &MIN,GLuint &MAX)
{
 // posledny prvok pola uplynutych ms
 mstotal=mshistory[MSHISTORY-1]; MIN=mstotal; MAX=mstotal;
 // ostatne prvky pola
 loopi(MSHISTORY-1) {
  mstotal+=mshistory[i];
  if (mshistory[i]>MIN) MIN=mshistory[i];
  if (mshistory[i]<MAX) MAX=mshistory[i];
 }
 ACT=(1000*MSHISTORY)/mstotal; MIN=1000/MIN; MAX=1000/MAX;
}

// aktualizacia casovaca ***************************************************
void CInterface::updatetimer(void)
{
 // aktualny casovy okamih (ms)
 actTime=(GLuint) SDL_GetTicks();
 // aktualny casovy okamih nemoze byt "starsi" ako povodny casovy okamih
 if (actTime<oldTime) actTime=oldTime;
 // pocet uplynutych milisekund
 millis=actTime-oldTime;
 // uprava poctu uplynutych milisekund
 fpsdelay=1000/MAXFPS-millis;
 if (fpsdelay<=0) fpserror=0; // ak MAXFPS=200 => ak millis>=5
 else {
  fpserror+=1000%MAXFPS; // ak MAXFPS=200 => fpserror+=0
  if (fpserror>=MAXFPS) { fpserror-=MAXFPS; fpsdelay++; }
  if (fpsdelay>0) { // ak millis<5
   SDL_Delay(fpsdelay); // pauza
   actTime+=(GLuint) fpsdelay; // aktualny casovy okamih (ms)
   millis=actTime-oldTime; // pocet uplynutych milisekund
  }
 }

 // uplynuty cas medzi aktualnym a povodnym casovym okamihom (s)
 dTime=((GLfloat) millis)/1000.0f;
 oldTime=actTime;

 // aktualizacia pola uplynutych ms
 mshistory[mspos++]=(GLuint) max(1,min(1000,(int) millis));
 // po naplneni pola uplynutych ms sa vypocita actfps, minfps a maxfps
 if (mspos>=MSHISTORY) {
  mspos=0;
  // aktualny pocet fps, minimalny a maximalny pocet fps
  getfps(actfps,minfps,maxfps);
  // INFO: fps
  //printf("%u %u %u\n",actfps,minfps,maxfps);
 }
}

// casovo zavisla uprava objektov, resp. sceny *****************************
void CInterface::OGL_idle(void)
{

}

// aktualizacia paramterov osvetlenia **************************************
void CInterface::update_lighting_params(void)
{
 glLightfv(GL_LIGHT0,GL_AMBIENT,lsrcparam.ambient.get_data());
 glLightfv(GL_LIGHT0,GL_DIFFUSE,lsrcparam.diffuse.get_data());
 glLightfv(GL_LIGHT0,GL_SPECULAR,lsrcparam.specular.get_data());
 glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,lsrcparam.spotDirection.get_data());
 glLightf(GL_LIGHT0,GL_SPOT_EXPONENT,lsrcparam.spotExponent);
 glLightf(GL_LIGHT0,GL_SPOT_CUTOFF,lsrcparam.spotCutoff);
 lsrcparam.spotCosCutoff=cos(DEG2RAD(lsrcparam.spotCutoff));
 lsrcparam.spotSinCutoff=sin(DEG2RAD(lsrcparam.spotCutoff));
 glLightf(GL_LIGHT0,GL_CONSTANT_ATTENUATION,lsrcparam.constantAttenuation);
 glLightf(GL_LIGHT0,GL_LINEAR_ATTENUATION,lsrcparam.linearAttenuation);
 glLightf(GL_LIGHT0,GL_QUADRATIC_ATTENUATION,lsrcparam.quadraticAttenuation);

 glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodelparam.ambient.get_data());

 glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,mparam.ambient.get_data());
 glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,mparam.diffuse.get_data());
 glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,mparam.specular.get_data());
 glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,mparam.emission.get_data());
 glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,mparam.shininess);
}

// vykreslenie OpenGL sceny ************************************************
void CInterface::OGL_draw(void)
{
 // vymazanie zasobnika farieb, hlbky a Stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

 // modelview matica bude upravena podla transformacnej matice Avatara
 mvmat=avatar.get_mv_mat();
 // uprava modelview matice
 glLoadMatrixf(mvmat.get_data());
 // umiestnenie svetelneho zdroja na danu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,lsrcparam.position.get_data());
 glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,lsrcparam.spotDirection.get_data());

 switch (but_text_id) {
  case 0: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          base_shader.on();
          // modelview matica
          base_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);
          objects_draw();
          base_shader.off();
          break;
  case 1: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          light_shader.on();
          // modelview matica
          light_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);
          objects_draw();
          light_shader.off();
          break;
  case 2: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          brick_shader.on();
          // modelview matica
          brick_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);
          objects_draw();
          brick_shader.off();
          break;
  case 3: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          toon_shader.on();
          // modelview matica
          toon_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);
          objects_draw();
          toon_shader.off();
          break;
  case 4: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          wave_shader.on();
          // aktualny casovy okamih
          wave_shader.uniform1F("at",actTime);
          // modelview matica
          wave_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[1]]);
          objects_draw();
          wave_shader.off();
          break;
  case 5: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          wood_shader.on();
          // modelview matica
          wood_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_3D,texture.tex_id[tid[3]]);
          objects_draw();
          wood_shader.off();
          break;
  case 6: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          nm_shader.on();
          // modelview matica
          nm_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textur
          glActiveTexture(GL_TEXTURE0);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[11]]);
          glActiveTexture(GL_TEXTURE1);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[12]]);
          objects_draw();
          glActiveTexture(GL_TEXTURE0);
          nm_shader.off();
          break;
  case 7: // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
          px_shader.on();
          // modelview matica
          px_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
          // pripojenie textur
          glActiveTexture(GL_TEXTURE0);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[13]]);
          glActiveTexture(GL_TEXTURE1);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[14]]);
          glActiveTexture(GL_TEXTURE2);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[15]]);
          objects_draw();
          glActiveTexture(GL_TEXTURE0);
          px_shader.off();
          break;
  case 8: // priprava reflection textury
          draw_water_scene(0);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_reflection]);
          glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,512,512);

          // priprava refraction textury
          glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
          draw_water_scene(1);
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_refraction]);
          glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,512,512);
          // priprava depth textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_depth]);
          glCopyTexImage2D(GL_TEXTURE_2D,0,GL_DEPTH_COMPONENT,0,0,512,512,0);

          // vykreslenie kompletnej sceny s vodou
          glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
          draw_water_scene(2);

          break;
  case 9: // 1. krok - vykreslenie sceny do FBO1 - depth, texture
          glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,fbo.getfbo1());
          glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
          glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

          dof_shader.on(0); // shader pre vykreslenie sceny a vypocet blur
          dof_shader.uniformmatrixFV(0,"modelview_mat",avatar.get_mv_mat());
          // pripojenie textury
          glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[16]]);
          // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
		  objects_draw();
          dof_shader.off();

          // ulozenie potrebnych atributov
          glPushAttrib(GL_DEPTH_BUFFER_BIT);
          glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky

          glMatrixMode(GL_PROJECTION); // projekcna matica
          glPushMatrix();              // ulozenie
          glLoadIdentity();            // reset

          // nastavenie pravouhleho premietania - vymeni Y
          glOrtho(0.0f,(GLfloat) scr_w,(GLfloat) scr_h,0.0f,-1.0f,1.0f);

          glMatrixMode(GL_MODELVIEW); // modelview matica
          glPushMatrix();             // ulozenie
          glLoadIdentity();           // reset

          objects_unbind(); // odpojenie buffrov

          glViewport(0,0,scr_w2,scr_h2); // polovicne rozmery okna

          // 2. krok - vykreslenie obsahu FBO1 do FBO2
          glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,fbo.getfbo2());

          glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT); // kresli do FBO2 - texture0
          glClear(GL_COLOR_BUFFER_BIT);

          dof_shader.on(1); // shader pre jednoduche vykreslenie stvoruholnika
          fbo.drawfbo1_attachment(); // vykresli obsah FBO1 - texture
          dof_shader.off();

          // 3. krok - Gausov filter v smere X
          glDrawBuffer(GL_COLOR_ATTACHMENT1_EXT); // kresli do FBO2 - texture1
          glClear(GL_COLOR_BUFFER_BIT);

          dof_shader.on(2); // aplikacia Gausovho filtra v smere X
          fbo.drawfbo2_attachment(0); // vykresli obsah FBO2 - texture0
          dof_shader.off();

          // 4. krok - Gausov filter v smere Y
          glDrawBuffer(GL_COLOR_ATTACHMENT2_EXT); // kresli do FBO2 - texture2
          glClear(GL_COLOR_BUFFER_BIT);

          dof_shader.on(3); // aplikacia Gausovho filtra v smere Y
          fbo.drawfbo2_attachment(1); // vykresli obsah FBO2 - texture1
          dof_shader.off();

          // 5. krok - vysledne vykreslenie
          glViewport(0,0,scr_w,scr_h); // plne rozmery okna

          // vykreslenie sceny do aplikaciou alokovaneho buffra
          glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,0);
          glDrawBuffer(GL_BACK);
          glActiveTexture(GL_TEXTURE0); // originalna scena
          glBindTexture(GL_TEXTURE_2D,fbo.getfbo1_tex_id()); // obsah FBO1 - texture
          glActiveTexture(GL_TEXTURE1); // Gausov filter
          glBindTexture(GL_TEXTURE_2D,fbo.getfbo2_tex_id(2)); // obsah FBO2 - texture2
          dof_shader.on(4); // zmiesanie originalnej sceny s Gaussian blur
          fbo.draw(); // vykresli stvoruholnik do celeho okna
          glActiveTexture(GL_TEXTURE0);
          dof_shader.off();

          // nastavenie povodnych ukazovatelov na udaje objektov
          objects_bind();
 
          glPopMatrix(); // obnova modelview matice
          glMatrixMode(GL_PROJECTION);
          glPopMatrix(); // obnova projection matice

          // aktualna je vzdy modelview matica
          glMatrixMode(GL_MODELVIEW);
          glPopAttrib(); // DEPTH_BUFFER_BIT

          break;
 }

 glColor4f(1.0f,1.0f,1.0f,1.0f); // biela
/*
 // vykreslenie TBN
 glPushAttrib(GL_TEXTURE_BIT);
 glDisable(GL_TEXTURE_2D);

 glBegin(GL_LINES);
  for (int i=0;i<4824;i+=3) {
   glColor4f(1.0f,0.0f,0.0f,1.0f);
   glVertex3f(v_a[i+0],v_a[i+1],v_a[i+2]);
   glVertex3f(v_a[i+0]+tan_a[i+0]/8.0f,v_a[i+1]+tan_a[i+1]/8.0f,v_a[i+2]+tan_a[i+2]/8.0f);
   glColor4f(0.0f,1.0f,0.0f,1.0f);
   glVertex3f(v_a[i+0],v_a[i+1],v_a[i+2]);
   glVertex3f(v_a[i+0]+bit_a[i+0]/8.0f,v_a[i+1]+bit_a[i+1]/8.0f,v_a[i+2]+bit_a[i+2]/8.0f);
   glColor4f(0.0f,0.0f,1.0f,1.0f);
   glVertex3f(v_a[i+0],v_a[i+1],v_a[i+2]);
   glVertex3f(v_a[i+0]+nor_a[i+0]/2.0f,v_a[i+1]+nor_a[i+1]/2.0f,v_a[i+2]+nor_a[i+2]/2.0f);
  }
 glEnd();

 glColor4f(1.0f,1.0f,1.0f,1.0f); // biela

 glPopAttrib();
*/
/*
 // vykreslenie svetelneho zdroja
 // billboarding
 v[0].set(0.3f,0.0f,0.0f);
 s.x=(v[0].x*mvmat[0]);
 s.y=(v[0].x*mvmat[4]);
 s.z=(v[0].x*mvmat[8]);
 v[0].set(0.0f,0.3f,0.0f);
 t.x=(v[0].y*mvmat[1]);
 t.y=(v[0].y*mvmat[5]);
 t.z=(v[0].y*mvmat[9]);
 v[0].set(-s.x-t.x,-s.y-t.y,-s.z-t.z); // 1. vrchol
 v[1].set( s.x-t.x, s.y-t.y, s.z-t.z); // 2. vrchol
 v[2].set( s.x+t.x, s.y+t.y, s.z+t.z); // 3. vrchol
 v[3].set(-s.x+t.x,-s.y+t.y,-s.z+t.z); // 4. vrchol

 glEnable(GL_BLEND);
 glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[10]]);
 glTranslatef(lsrcparam.position.x,lsrcparam.position.y,lsrcparam.position.z);
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3fv(v[0].get_data());
  glTexCoord2f(1.0f,0.0f); glVertex3fv(v[1].get_data());
  glTexCoord2f(1.0f,1.0f); glVertex3fv(v[2].get_data());
  glTexCoord2f(0.0f,1.0f); glVertex3fv(v[3].get_data());
 glEnd();
 glDisable(GL_BLEND);
*/
 glTranslatef(lsrcparam.position.x,lsrcparam.position.y,lsrcparam.position.z);
 glRotatef(lA,0.0f,1.0f,0.0f);
 glRotatef(lB,1.0f,0.0f,0.0f);
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[10]]);
 glBegin(GL_QUADS);
  glTexCoord2f(0.12f,0.055f); glVertex3f(-0.1f,-0.1f, 0.2f); glTexCoord2f(0.35f,0.055f); glVertex3f( 0.1f,-0.1f, 0.2f);
  glTexCoord2f(0.35f,0.282f); glVertex3f( 0.1f, 0.1f, 0.2f); glTexCoord2f(0.12f,0.282f); glVertex3f(-0.1f, 0.1f, 0.2f);

  glTexCoord2f(0.12f,0.0f);  glVertex3f(-0.1f,-0.1f, 0.2f); glTexCoord2f(0.35f,0.0f);  glVertex3f(-0.1f, 0.1f, 0.2f);
  glTexCoord2f(0.35f,0.35f); glVertex3f(-0.1f, 0.1f,-0.2f); glTexCoord2f(0.12f,0.35f); glVertex3f(-0.1f,-0.1f,-0.2f);
  glTexCoord2f(0.12f,0.0f);  glVertex3f( 0.1f, 0.1f, 0.2f); glTexCoord2f(0.35f,0.0f);  glVertex3f( 0.1f,-0.1f, 0.2f);
  glTexCoord2f(0.35f,0.35f); glVertex3f( 0.1f,-0.1f,-0.2f); glTexCoord2f(0.12f,0.35f); glVertex3f( 0.1f, 0.1f,-0.2f);
  glTexCoord2f(0.12f,0.0f);  glVertex3f(-0.1f, 0.1f, 0.2f); glTexCoord2f(0.35f,0.0f);  glVertex3f( 0.1f, 0.1f, 0.2f);
  glTexCoord2f(0.35f,0.35f); glVertex3f( 0.1f, 0.1f,-0.2f); glTexCoord2f(0.12f,0.35f); glVertex3f(-0.1f, 0.1f,-0.2f);
  glTexCoord2f(0.12f,0.0f);  glVertex3f( 0.1f,-0.1f, 0.2f); glTexCoord2f(0.35f,0.0f);  glVertex3f(-0.1f,-0.1f, 0.2f);
  glTexCoord2f(0.35f,0.35f); glVertex3f(-0.1f,-0.1f,-0.2f); glTexCoord2f(0.12f,0.35f); glVertex3f( 0.1f,-0.1f,-0.2f);

  glTexCoord2f(0.12f,0.35f); glVertex3f(-0.1f,  0.1f, -0.2f); glTexCoord2f(0.35f,0.35f); glVertex3f( 0.1f,  0.1f, -0.2f);
  glTexCoord2f(0.48f,0.5f);  glVertex3f( 0.15f, 0.15f,-0.4f); glTexCoord2f(0.0f,0.5f);   glVertex3f(-0.15f, 0.15f,-0.4f);
  glTexCoord2f(0.12f,0.35f); glVertex3f( 0.1f, -0.1f, -0.2f); glTexCoord2f(0.35f,0.35f); glVertex3f(-0.1f, -0.1f, -0.2f);
  glTexCoord2f(0.48f,0.5f);  glVertex3f(-0.15f,-0.15f,-0.4f); glTexCoord2f(0.0f,0.5f);   glVertex3f( 0.15f,-0.15f,-0.4f);
  glTexCoord2f(0.35f,0.35f); glVertex3f(-0.1f,  0.1f, -0.2f); glTexCoord2f(0.48f,0.5f);  glVertex3f(-0.15f, 0.15f,-0.4f);
  glTexCoord2f(0.0f,0.5f);   glVertex3f(-0.15f,-0.15f,-0.4f); glTexCoord2f(0.12f,0.35f); glVertex3f(-0.1f, -0.1f, -0.2f);
  glTexCoord2f(0.12f,0.35f); glVertex3f( 0.1f,  0.1f, -0.2f); glTexCoord2f(0.35f,0.35f); glVertex3f( 0.1f, -0.1f, -0.2f);
  glTexCoord2f(0.48f,0.5f);  glVertex3f( 0.15f,-0.15f,-0.4f); glTexCoord2f(0.0f,0.5f);   glVertex3f( 0.15f, 0.15f,-0.4f);

  glTexCoord2f(0.5f,0.5f); glVertex3f(-0.15f, 0.15f,-0.4f); glTexCoord2f(1.0f,0.5f); glVertex3f( 0.15f, 0.15f,-0.4f);
  glTexCoord2f(1.0f,1.0f); glVertex3f( 0.15f,-0.15f,-0.4f); glTexCoord2f(0.5f,1.0f); glVertex3f(-0.15f,-0.15f,-0.4f);
 glEnd();
 glPushAttrib(GL_TEXTURE_BIT);
 glDisable(GL_TEXTURE_2D);
 glBegin(GL_LINES);
  glColor4f(0.5f,0.5f,0.5f,1.0f);
  glVertex3f(0.0f,0.0f,-0.4f); glVertex3f(0.0f,0.0f,-5.0f);
  glColor4f(1.0f,1.0f,1.0f,1.0f);
  glVertex3f(0.0f,0.0f,-0.4f); glVertex3f(5.0f*lsrcparam.spotSinCutoff,0.0f,-5.0f*lsrcparam.spotCosCutoff);
  glVertex3f(0.0f,0.0f,-0.4f); glVertex3f(-5.0f*lsrcparam.spotSinCutoff,0.0f,-5.0f*lsrcparam.spotCosCutoff);
  glVertex3f(0.0f,0.0f,-0.4f); glVertex3f(0.0f,5.0f*lsrcparam.spotSinCutoff,-5.0f*lsrcparam.spotCosCutoff);
  glVertex3f(0.0f,0.0f,-0.4f); glVertex3f(0.0f,-5.0f*lsrcparam.spotSinCutoff,-5.0f*lsrcparam.spotCosCutoff);
 glEnd();
 glPopAttrib();

 // vykreslenie head-up display
 drawhud();
}

// viacnasobne vykreslenie sceny s vodou ***********************************
void CInterface::draw_water_scene(int PASS)
{
 if (PASS<2) glViewport(0,0,512,512);
 else glViewport(0,0,(GLfloat) scr_w,(GLfloat) scr_h);

 if (PASS==0) { // reflection texture
  glCullFace(GL_FRONT);

  wtr.calc_water_plane(0.0f,-1.0f,0.0f,1.0f);
  glClipPlane(GL_CLIP_PLANE0,wtr.water_plane);
  glEnable(GL_CLIP_PLANE0);

  avatar.mv_mat_translate(0.0f,2.0f*wtr.water_level,0.0f);
  avatar.mv_mat_scale(1.0f,-1.0f,1.0f);
 }

 if (PASS==1) { // refraction texture
  wtr.calc_water_plane(0.0f,-1.0f,0.0f,1.0f);
  glClipPlane(GL_CLIP_PLANE0,wtr.water_plane);
  glEnable(GL_CLIP_PLANE0);
 }

 // vykreslenie objektov - kocka, gula, valec, ihlan, plochy
 px_shader.on();
 // modelview matica
 px_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
 // pripojenie textur
 glActiveTexture(GL_TEXTURE0);
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[13]]);
 glActiveTexture(GL_TEXTURE1);
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[14]]);
 glActiveTexture(GL_TEXTURE2);
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[15]]);
 objects_draw();
 glActiveTexture(GL_TEXTURE0);
 px_shader.off();

 if (PASS==0) {
  avatar.mv_mat_scale(1.0f,-1.0f,1.0f);
  avatar.mv_mat_translate(0.0f,-2.0f*wtr.water_level,0.0f);

  glDisable(GL_CLIP_PLANE0);

  glCullFace(GL_BACK);
 }

 if (PASS==1) {
  glDisable(GL_CLIP_PLANE0);
 }

 if (PASS==2) { // kompletna scena s vodou
  objects_unbind(); // odpojenie buffrov

  // vykreslenie objektu predstavujuceho hladinu vody
  water_shader.on();
  // modelview matica
  water_shader.uniformmatrixFV("modelview_mat",avatar.get_mv_mat());
  // pripojenie textur
  glActiveTexture(GL_TEXTURE0);
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_reflection]);
  glActiveTexture(GL_TEXTURE1);
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_refraction]);
  glActiveTexture(GL_TEXTURE2);
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_depth]);
  glActiveTexture(GL_TEXTURE3);
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_nm]);
  glActiveTexture(GL_TEXTURE4);
  glBindTexture(GL_TEXTURE_2D,texture.tex_id[wtr.tex_dudv]);
  // aktualny casovy okamih
  water_shader.uniform1F("at",actTime/10000.0f);

  wtr.draw();

  glActiveTexture(GL_TEXTURE0);
  water_shader.off();

  // nastavenie povodnych ukazovatelov na udaje objektov
  objects_bind();
 }
}

// vykreslenie head-up display *********************************************
void CInterface::drawhud(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_DEPTH_BUFFER_BIT);

 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glEnable(GL_BLEND);       // zapnutie priesvitnosti

 glMatrixMode(GL_PROJECTION); // projekcna matica
 glPushMatrix();              // ulozenie
 glLoadIdentity();            // reset

 // nastavenie pravouhleho premietania - vymeni Y
 glOrtho(0.0f,(GLfloat) scr_w,(GLfloat) scr_h,0.0f,-1.0f,1.0f);

 glMatrixMode(GL_MODELVIEW); // modelview matica
 glPushMatrix();             // ulozenie
 glLoadIdentity();           // reset

 // funkcia pre vypocet priesvitnosti
 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_ALPHA);

 // vykreslenie menu -------------------------------------------------------

 // pripojenie prazdnej textury
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[0]]);
 // farba (priesvitnost) menu
 glColor4f(1.0f,1.0f,1.0f,0.1f);
 // stvoruholnik
 glBegin(GL_QUADS); // glOrtho() vymeni Y
  glVertex3f(5.0f,  5.0f,               0.0f);
  glVertex3f(5.0f,  (GLfloat) (scr_h-5),0.0f);
  glVertex3f(150.f, (GLfloat) (scr_h-5),0.0f);
  glVertex3f(150.0f,5.0f,               0.0f);
 glEnd();

 // vykreslenie tlacidiel
 loopi(BUTNUM) but[i].draw();

 // vypis textu ------------------------------------------------------------

 // pripojenie textury s fontom
 glBindTexture(GL_TEXTURE_2D,text.tex_id);

 // velkost textu
 text.scale(0.5f,0.5f);
 // farba textu
 glColor4f(0.8f,0.8f,0.0f,1.0f);
 // pocet fps
 text.drawf(15.0f,15.0f,"%u",actfps);
 // poloha Avatara
 text.drawf(15.0f,45.0f,"%.1f %.1f %.1f",-avatar.actpos.x,-avatar.actpos.y,-avatar.actpos.z);

 // velkost textu
 text.scale(0.4f,0.4f);
 // parametre svetelneho zdroja
 glColor4f(0.8f,0.8f,0.8f,1.0f); text.drawf(25.0f,120.0f,"Light source");
 glColor4f(0.8f,0.8f,0.0f,1.0f);
 text.drawf(25.0f,154.0,"ambient:");
 text.drawf(25.0f,229.0,"diffuse:");
 text.drawf(25.0f,304.0f,"specular:");
 text.drawf(25.0f,379.0f,"position:");
 text.drawf(25.0f,454.0f,"spotDirection:");
 text.drawf(25.0f,529.0f,"spotExponent:");
 text.drawf(25.0f,604.0f,"spotCutoff:");
 text.drawf(25.0f,679.0f,"constantAtt:");
 text.drawf(25.0f,754.0f,"linearAtt:");
 text.drawf(25.0f,829.0f,"quadraticAtt:");
 text.drawf(25.0f,904.0f,"global ambient:");
 // parametre materialu
 glColor4f(0.8f,0.8f,0.8f,1.0f); text.drawf(25.0f,995.0f,"Material");
 glColor4f(0.8f,0.8f,0.0f,1.0f);
 text.drawf(25.0f,1029.0f,"ambient:");
 text.drawf(25.0f,1104.0f,"diffuse:");
 text.drawf(25.0f,1179.0f,"specular:");
 text.drawf(25.0f,1254.0f,"emission:");
 text.drawf(25.0f,1329.0f,"shininess:");
 // pouzity shader
 glColor4f(0.8f,0.8f,0.8f,1.0f); text.drawf(25.0f,1404.0f,"Shader");

 // vykreslenie zameriavacieho kriza ---------------------------------------

 // funkcia pre vypocet priesvitnosti
 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);

 // farba zameriavacieho kriza
 glColor4f(0.5f,0.5f,0.5f,1.0f);
 // pripojenie textury zameriavacieho kriza
 glBindTexture(GL_TEXTURE_2D,texture.tex_id[tid[2]]);

 glPushMatrix(); // ulozenie modelview matice
 // zameriavaci kriz na stred okna
 glTranslatef((GLfloat) scr_w2-crosshairSize/2.0f,
              (GLfloat) scr_h2-crosshairSize/2.0f,0.0f);
 // stvoruholnik
 glBegin(GL_QUADS); // glOrtho() vymeni Y
  glTexCoord2f(0.0f,0.0f); glVertex3f(0.0f,0.0f,0.0f);
  glTexCoord2f(0.0f,1.0f); glVertex3f(0.0f,crosshairSize,0.0f);
  glTexCoord2f(1.0f,1.0f); glVertex3f(crosshairSize,crosshairSize,0.0f);
  glTexCoord2f(1.0f,0.0f); glVertex3f(crosshairSize,0.0f,0.0f);
 glEnd();
 glPopMatrix(); // obnova modelview matice

 glColor4f(1.0f,1.0f,1.0f,1.0f); // biela

 glPopMatrix(); // obnova modelview matice
 glMatrixMode(GL_PROJECTION);
 glPopMatrix(); // obnova projection matice

 // aktualna je vzdy modelview matica
 glMatrixMode(GL_MODELVIEW);

 glDisable(GL_BLEND); // vypnutie priesvitnosti
 glPopAttrib();       // vyber ulozenych atributov - DEPTH_BUFFER
}

// vypocet TBN vektorov ****************************************************
void CInterface::TBN(int E,int Q,int W)
{
 vec3 v0,v1,v2;
 vec2 v0t,v1t,v2t;
 vec3 tan,bit,nor;

 v0.set(v_a[E+Q+0],v_a[E+Q+1],v_a[E+Q+2]); // 3 vrcholy
 v1.set(v_a[E+Q+3],v_a[E+Q+4],v_a[E+Q+5]);
 v2.set(v_a[E+Q+6],v_a[E+Q+7],v_a[E+Q+8]);
 if (E==0) { // kocka
  v0t.set(t_a[W+0],t_a[W+1]);              // 3 texturovacie suradnice
  v1t.set(t_a[W+2],t_a[W+3]);
  v2t.set(t_a[W+4],t_a[W+5]);
 }
 else {      // ostatne objekty
  v0t.set(t_a[72+W+0],t_a[72+W+1]);        // 3 texturovacie suradnice
  v1t.set(t_a[72+W+2],t_a[72+W+3]);
  v2t.set(t_a[72+W+4],t_a[72+W+5]);
 }
 mm.calc_tbn(v0,v1,v2,v0t,v1t,v2t,tan,bit,nor); // vypocet TBN
 tan_a[E+Q+0]=tan.x; tan_a[E+Q+1]=tan.y; tan_a[E+Q+2]=tan.z; // 1. vrchol
 tan_a[E+Q+3]=tan.x; tan_a[E+Q+4]=tan.y; tan_a[E+Q+5]=tan.z; // 2. vrchol
 tan_a[E+Q+6]=tan.x; tan_a[E+Q+7]=tan.y; tan_a[E+Q+8]=tan.z; // 3. vrchol
 bit_a[E+Q+0]=bit.x; bit_a[E+Q+1]=bit.y; bit_a[E+Q+2]=bit.z; // 1. vrchol
 bit_a[E+Q+3]=bit.x; bit_a[E+Q+4]=bit.y; bit_a[E+Q+5]=bit.z; // 2. vrchol
 bit_a[E+Q+6]=bit.x; bit_a[E+Q+7]=bit.y; bit_a[E+Q+8]=bit.z; // 3. vrchol
 nor_a[E+Q+0]=nor.x; nor_a[E+Q+1]=nor.y; nor_a[E+Q+2]=nor.z; // 1. vrchol
 nor_a[E+Q+3]=nor.x; nor_a[E+Q+4]=nor.y; nor_a[E+Q+5]=nor.z; // 2. vrchol
 nor_a[E+Q+6]=nor.x; nor_a[E+Q+7]=nor.y; nor_a[E+Q+8]=nor.z; // 3. vrchol
}

// priprava objektov - kocka, gula, valec, ihlan, plochy *******************
void CInterface::objects_prepare(void)
{
 // pomocne premenne
 int q=0,w=0,e=0,f=0,r=0,s=0,t=0;
 float alfa[2],beta[2];
 float x[4],y[2],z[4];
 vec3 n[7];

 e=108; // pocet udajov patriacich kocke

 // kocka ------------------------------------------------------------------
 float tmp[]={-4.0f, 0.0f, 4.0f,-2.0f, 0.0f, 4.0f,-2.0f, 2.0f, 4.0f,
              -4.0f, 0.0f, 4.0f,-2.0f, 2.0f, 4.0f,-4.0f, 2.0f, 4.0f,
              -2.0f, 0.0f, 4.0f,-2.0f, 0.0f, 2.0f,-2.0f, 2.0f, 2.0f,
              -2.0f, 0.0f, 4.0f,-2.0f, 2.0f, 2.0f,-2.0f, 2.0f, 4.0f,
              -2.0f, 0.0f, 2.0f,-4.0f, 0.0f, 2.0f,-4.0f, 2.0f, 2.0f,
              -2.0f, 0.0f, 2.0f,-4.0f, 2.0f, 2.0f,-2.0f, 2.0f, 2.0f,
              -4.0f, 0.0f, 2.0f,-4.0f, 0.0f, 4.0f,-4.0f, 2.0f, 4.0f,
              -4.0f, 0.0f, 2.0f,-4.0f, 2.0f, 4.0f,-4.0f, 2.0f, 2.0f,
              -4.0f, 2.0f, 4.0f,-2.0f, 2.0f, 4.0f,-2.0f, 2.0f, 2.0f,
              -4.0f, 2.0f, 4.0f,-2.0f, 2.0f, 2.0f,-4.0f, 2.0f, 2.0f,
              -4.0f, 0.0f, 2.0f,-2.0f, 0.0f, 2.0f,-2.0f, 0.0f, 4.0f,
              -4.0f, 0.0f, 2.0f,-2.0f, 0.0f, 4.0f,-4.0f, 0.0f, 4.0f};
 loopi(e) v_a[i]=tmp[i];
 // farby
 for (int i=0;i<e;i+=3) { c_a[i]=0.5f; c_a[i+1]=0.0f; c_a[i+2]=0.0f; }
 // normaly
 float tmp1[]={ 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f,
                0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f,
                1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
                1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f,
                0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f,
                0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f,
               -1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,
               -1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,
                0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f,
                0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f,
                0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f,
                0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f, 0.0f,-1.0f, 0.0f};
 loopi(e) nor_a[i]=tmp1[i];
 // texturovacie suradnice
 float tmp2[]={0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f,
               0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f};
 loopi(72) t_a[i]=tmp2[i];
 // TBN pre kocku
 for (int i=0,j=0;i<e;i+=9,j+=6) TBN(0,i,j);

 f=e+q; // zaciatok udajov gule

 // gula -------------------------------------------------------------------
 loopj(180) {
  if (j%segs) continue;
  loopi(360) {
   if (i%segs) continue;

   alfa[0]=DEG2RAD((float) i);        beta[0]=DEG2RAD((float) j);
   alfa[1]=DEG2RAD((float) (i+segs)); beta[1]=DEG2RAD((float) (j+segs));

   x[0]=3.0f+cos(alfa[0])*sin(beta[0]); x[1]=3.0f+cos(alfa[1])*sin(beta[0]);
   x[2]=3.0f+cos(alfa[1])*sin(beta[1]); x[3]=3.0f+cos(alfa[0])*sin(beta[1]);
   y[0]=1.0f+cos(beta[0]); y[1]=1.0f+cos(beta[1]);
   z[0]=3.0f+sin(alfa[0])*sin(beta[0]); z[1]=3.0f+sin(alfa[1])*sin(beta[0]);
   z[2]=3.0f+sin(alfa[1])*sin(beta[1]); z[3]=3.0f+sin(alfa[0])*sin(beta[1]);

   v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
   v_a[e+q+3]=x[1]; v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1]; // 2. vrchol
   v_a[e+q+6]=x[2]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[2]; // 3. vrchol
   c_a[e+q+0]=0.0f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
   c_a[e+q+3]=0.0f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
   c_a[e+q+6]=0.0f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
   t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=(float) j/90.0f;        // 1. vrchol
   t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=(float) j/90.0f;        // 2. vrchol
   t_a[72+w+4]=(float) (i+segs)/90.0f; t_a[72+w+5]=(float) (j+segs)/90.0f; // 3. vrchol
   TBN(e,q,w);
   q+=9; w+=6;

   v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
   v_a[e+q+3]=x[2]; v_a[e+q+4]=y[1]; v_a[e+q+5]=z[2]; // 2. vrchol
   v_a[e+q+6]=x[3]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[3]; // 3. vrchol
   c_a[e+q+0]=0.0f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
   c_a[e+q+3]=0.0f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
   c_a[e+q+6]=0.0f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
   t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=(float) j/90.0f;        // 1. vrchol
   t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=(float) (j+segs)/90.0f; // 2. vrchol
   t_a[72+w+4]=(float) i/90.0f;        t_a[72+w+5]=(float) (j+segs)/90.0f; // 3. vrchol
   TBN(e,q,w);
   q+=9; w+=6;
  }
 }
 // vyhladenie normal
 t=18*(360/segs);
 loopj(180) {
  if ((j==0) || (j%segs)) continue;
  loopi(360) {
   if (i%segs) continue;
   r=i+j*(360/segs);
   s=i+(j-segs)*(360/segs);
   if (i==0) {
    n[0].set(nor_a[f+r+0],   nor_a[f+r+1],   nor_a[f+r+2]);
    n[1].set(nor_a[f+r+9],   nor_a[f+r+10],  nor_a[f+r+11]);
    n[2].set(nor_a[f+r+t-15],nor_a[f+r+t-14],nor_a[f+r+t-13]);
    n[3].set(nor_a[f+s+t-6], nor_a[f+s+t-5], nor_a[f+s+t-4]);
    n[4].set(nor_a[f+s+t-12],nor_a[f+s+t-11],nor_a[f+s+t-10]);
    n[5].set(nor_a[f+s+15],  nor_a[f+s+16],  nor_a[f+s+17]);
    n[6]=(n[0]+n[1]+n[2]+n[3]+n[4]+n[5])/6.0f; n[6].normalize();
    nor_a[f+r+0]=   n[6].x; nor_a[f+r+1]=   n[6].y; nor_a[f+r+2]=   n[6].z;
    nor_a[f+r+9]=   n[6].x; nor_a[f+r+10]=  n[6].y; nor_a[f+r+11]=  n[6].z;
    nor_a[f+r+t-15]=n[6].x; nor_a[f+r+t-14]=n[6].y; nor_a[f+r+t-13]=n[6].z;
    nor_a[f+s+t-6]= n[6].x; nor_a[f+s+t-5]= n[6].y; nor_a[f+s+t-4]= n[6].z;
    nor_a[f+s+t-12]=n[6].x; nor_a[f+s+t-11]=n[6].y; nor_a[f+s+t-10]=n[6].z;
    nor_a[f+s+15]=  n[6].x; nor_a[f+s+16]=  n[6].y; nor_a[f+s+17]=  n[6].z;
   }
   else {
    n[0].set(nor_a[f+r+0], nor_a[f+r+1], nor_a[f+r+2]);
    n[1].set(nor_a[f+r+9], nor_a[f+r+10],nor_a[f+r+11]);
    n[2].set(nor_a[f+r-15],nor_a[f+r-14],nor_a[f+r-13]);
    n[3].set(nor_a[f+s-6], nor_a[f+s-5], nor_a[f+s-4]);
    n[4].set(nor_a[f+s-12],nor_a[f+s-11],nor_a[f+s-10]);
    n[5].set(nor_a[f+s+15],nor_a[f+s+16],nor_a[f+s+17]);
    n[6]=(n[0]+n[1]+n[2]+n[3]+n[4]+n[5])/6.0f; n[6].normalize();
    nor_a[f+r+0]= n[6].x; nor_a[f+r+1]= n[6].y; nor_a[f+r+2]= n[6].z;
    nor_a[f+r+9]= n[6].x; nor_a[f+r+10]=n[6].y; nor_a[f+r+11]=n[6].z;
    nor_a[f+r-15]=n[6].x; nor_a[f+r-14]=n[6].y; nor_a[f+r-13]=n[6].z;
    nor_a[f+s-6]= n[6].x; nor_a[f+s-5]= n[6].y; nor_a[f+s-4]= n[6].z;
    nor_a[f+s-12]=n[6].x; nor_a[f+s-11]=n[6].y; nor_a[f+s-10]=n[6].z;
    nor_a[f+s+15]=n[6].x; nor_a[f+s+16]=n[6].y; nor_a[f+s+17]=n[6].z;
   }
  }
 }

 f=e+q; // zaciatok udajov valca

 // valec ------------------------------------------------------------------
 loopi(360) {
  if (i%segs) continue;

  alfa[0]=DEG2RAD((float) i);
  alfa[1]=DEG2RAD((float) (i+segs));

  x[0]=3.0f+cos(alfa[0]); x[1]=3.0f+cos(alfa[1]);
  y[0]=0.0f; y[1]=2.0f;
  z[0]=-3.0f-sin(alfa[0]); z[1]=-3.0f-sin(alfa[1]);

  v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1]; // 2. vrchol
  v_a[e+q+6]=x[1]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[1]; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i+segs)/90.0f; t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;

  v_a[e+q+0]=x[0]; v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0]; // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=y[1]; v_a[e+q+5]=z[1]; // 2. vrchol
  v_a[e+q+6]=x[0]; v_a[e+q+7]=y[1]; v_a[e+q+8]=z[0]; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=1.0f; // 2. vrchol
  t_a[72+w+4]=(float) i/90.0f;        t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;

  v_a[e+q+0]=x[0]; v_a[e+q+1]=2.0f; v_a[e+q+2]=z[0];  // 1. vrchol
  v_a[e+q+3]=x[1]; v_a[e+q+4]=2.0f; v_a[e+q+5]=z[1];  // 2. vrchol
  v_a[e+q+6]=3.0f; v_a[e+q+7]=2.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i/90);         t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;

  v_a[e+q+0]=x[1]; v_a[e+q+1]=0.0f; v_a[e+q+2]=z[1];  // 1. vrchol
  v_a[e+q+3]=x[0]; v_a[e+q+4]=0.0f; v_a[e+q+5]=z[0];  // 2. vrchol
  v_a[e+q+6]=3.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.0f; c_a[e+q+1]=0.0f; c_a[e+q+2]=0.5f; // 1. vrchol
  c_a[e+q+3]=0.0f; c_a[e+q+4]=0.0f; c_a[e+q+5]=0.5f; // 2. vrchol
  c_a[e+q+6]=0.0f; c_a[e+q+7]=0.0f; c_a[e+q+8]=0.5f; // 3. vrchol
  t_a[72+w+0]=(float) (i+segs)/90.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) i/90.0f;        t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i/90);         t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;
 }
 // vyhladenie normal
 t=36*(360/segs);
 for (int i=0;i<36*(360/segs);i+=36) {
  if (i==0) {
   n[0].set(nor_a[f+i+0],   nor_a[f+i+1],   nor_a[f+i+2]);
   n[1].set(nor_a[f+i+9],   nor_a[f+i+10],  nor_a[f+i+11]);
   n[2].set(nor_a[f+i+t-33],nor_a[f+i+t-32],nor_a[f+i+t-31]);
   n[3].set(nor_a[f+i+15],  nor_a[f+i+16],  nor_a[f+i+17]);
   n[4].set(nor_a[f+i+t-30],nor_a[f+i+t-29],nor_a[f+i+t-28]);
   n[5].set(nor_a[f+i+t-24],nor_a[f+i+t-23],nor_a[f+i+t-22]);
   n[6]=(n[0]+n[1]+n[2])/3.0f; n[6].normalize();
   nor_a[f+i+0]=   n[6].x; nor_a[f+i+1]=   n[6].y; nor_a[f+i+2]=   n[6].z;
   nor_a[f+i+9]=   n[6].x; nor_a[f+i+10]=  n[6].y; nor_a[f+i+11]=  n[6].z;
   nor_a[f+i+t-33]=n[6].x; nor_a[f+i+t-32]=n[6].y; nor_a[f+i+t-31]=n[6].z;
   n[6]=(n[3]+n[4]+n[5])/3.0f; n[6].normalize();
   nor_a[f+i+15]=  n[6].x; nor_a[f+i+16]=  n[6].y; nor_a[f+i+17]=  n[6].z;
   nor_a[f+i+t-30]=n[6].x; nor_a[f+i+t-29]=n[6].y; nor_a[f+i+t-28]=n[6].z;
   nor_a[f+i+t-24]=n[6].x; nor_a[f+i+t-23]=n[6].y; nor_a[f+i+t-22]=n[6].z;
  }
  else {
   n[0].set(nor_a[f+i+0], nor_a[f+i+1], nor_a[f+i+2]);
   n[1].set(nor_a[f+i+9], nor_a[f+i+10],nor_a[f+i+11]);
   n[2].set(nor_a[f+i-33],nor_a[f+i-32],nor_a[f+i-31]);
   n[3].set(nor_a[f+i+15],nor_a[f+i+16],nor_a[f+i+17]);
   n[4].set(nor_a[f+i-30],nor_a[f+i-29],nor_a[f+i-28]);
   n[5].set(nor_a[f+i-24],nor_a[f+i-23],nor_a[f+i-22]);
   n[6]=(n[0]+n[1]+n[2])/3.0f; n[6].normalize();
   nor_a[f+i+0]= n[6].x; nor_a[f+i+1]= n[6].y; nor_a[f+i+2]= n[6].z;
   nor_a[f+i+9]= n[6].x; nor_a[f+i+10]=n[6].y; nor_a[f+i+11]=n[6].z;
   nor_a[f+i-33]=n[6].x; nor_a[f+i-32]=n[6].y; nor_a[f+i-31]=n[6].z;
   n[6]=(n[3]+n[4]+n[5])/3.0f; n[6].normalize();
   nor_a[f+i+15]=n[6].x; nor_a[f+i+16]=n[6].y; nor_a[f+i+17]=n[6].z;
   nor_a[f+i-30]=n[6].x; nor_a[f+i-29]=n[6].y; nor_a[f+i-28]=n[6].z;
   nor_a[f+i-24]=n[6].x; nor_a[f+i-23]=n[6].y; nor_a[f+i-22]=n[6].z;
  }
 }

 f=e+q; // zaciatok udajov ihlanu

 // ihlan ------------------------------------------------------------------
 loopi(360) {
  if (i%segs) continue;

  alfa[0]=DEG2RAD((float) i);
  alfa[1]=DEG2RAD((float) (i+segs));

  x[0]=-3.0f+cos(alfa[0]); x[1]=-3.0f+cos(alfa[1]);
  y[0]=0.0f; y[1]=2.0f;
  z[0]=-3.0f-sin(alfa[0]); z[1]=-3.0f-sin(alfa[1]);

  v_a[e+q+0]=x[0];  v_a[e+q+1]=y[0]; v_a[e+q+2]=z[0];  // 1. vrchol
  v_a[e+q+3]=x[1];  v_a[e+q+4]=y[0]; v_a[e+q+5]=z[1];  // 2. vrchol
  v_a[e+q+6]=-3.0f; v_a[e+q+7]=y[1]; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.5f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
  c_a[e+q+3]=0.5f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
  c_a[e+q+6]=0.5f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=(float) i/90.0f;        t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) (i+segs)/90.0f; t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i/90);         t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;

  v_a[e+q+0]=x[1];  v_a[e+q+1]=0.0f; v_a[e+q+2]=z[1];  // 1. vrchol
  v_a[e+q+3]=x[0];  v_a[e+q+4]=0.0f; v_a[e+q+5]=z[0];  // 2. vrchol
  v_a[e+q+6]=-3.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-3.0f; // 3. vrchol
  c_a[e+q+0]=0.5f; c_a[e+q+1]=0.5f; c_a[e+q+2]=0.0f; // 1. vrchol
  c_a[e+q+3]=0.5f; c_a[e+q+4]=0.5f; c_a[e+q+5]=0.0f; // 2. vrchol
  c_a[e+q+6]=0.5f; c_a[e+q+7]=0.5f; c_a[e+q+8]=0.0f; // 3. vrchol
  t_a[72+w+0]=(float) (i+segs)/90.0f; t_a[72+w+1]=0.0f; // 1. vrchol
  t_a[72+w+2]=(float) i/90.0f;        t_a[72+w+3]=0.0f; // 2. vrchol
  t_a[72+w+4]=(float) (i/90);         t_a[72+w+5]=1.0f; // 3. vrchol
  TBN(e,q,w);
  q+=9; w+=6;
 }
 // vyhladenie normal
 t=18*(360/segs);
 for (int i=0;i<18*(360/segs);i+=18) {
  if (i==0) {
   n[0].set(nor_a[f+i+0],   nor_a[f+i+1],   nor_a[f+i+2]);
   n[1].set(nor_a[f+i+t-15],nor_a[f+i+t-14],nor_a[f+i+t-13]);
   n[6]=(n[0]+n[1])/2.0f; n[6].normalize();
   nor_a[f+i+0]=   n[6].x; nor_a[f+i+1]=   n[6].y; nor_a[f+i+2]=   n[6].z;
   nor_a[f+i+t-15]=n[6].x; nor_a[f+i+t-14]=n[6].y; nor_a[f+i+t-13]=n[6].z;
   nor_a[f+i+6]=   0.0f; nor_a[f+i+7]=   1.0f; nor_a[f+i+8]=   0.0f;
   nor_a[f+i+t-12]=0.0f; nor_a[f+i+t-11]=1.0f; nor_a[f+i+t-10]=0.0f;
  }
  else {
   n[0].set(nor_a[f+i+0], nor_a[f+i+1], nor_a[f+i+2]);
   n[1].set(nor_a[f+i-15],nor_a[f+i-14],nor_a[f+i-13]);
   n[2].set(nor_a[f+i+6], nor_a[f+i+7], nor_a[f+i+8]);
   n[3].set(nor_a[f+i-12],nor_a[f+i-11],nor_a[f+i-10]);
   n[6]=(n[0]+n[1])/2.0f; n[6].normalize();
   nor_a[f+i+0]= n[6].x; nor_a[f+i+1]= n[6].y; nor_a[f+i+2]= n[6].z;
   nor_a[f+i-15]=n[6].x; nor_a[f+i-14]=n[6].y; nor_a[f+i-13]=n[6].z;
   nor_a[f+i+6]= 0.0f; nor_a[f+i+7]= 1.0f; nor_a[f+i+8]= 0.0f;
   nor_a[f+i-12]=0.0f; nor_a[f+i-11]=1.0f; nor_a[f+i-10]=0.0f;
  }
 }

 // plochy -----------------------------------------------------------------
 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f; v_a[e+q+2]= 5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=0.0f; v_a[e+q+5]= 5.0f; // 2. vrchol
 v_a[e+q+6]= 5.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=0.0f; // 2. vrchol
 t_a[72+w+4]=1.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 TBN(e,q,w);
 q+=9; w+=6;

 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f; v_a[e+q+2]= 5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=0.0f; v_a[e+q+5]=-5.0f; // 2. vrchol
 v_a[e+q+6]=-5.0f; v_a[e+q+7]=0.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=1.0f; // 2. vrchol
 t_a[72+w+4]=0.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 TBN(e,q,w);
 q+=9; w+=6;

 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f;  v_a[e+q+2]=-5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=0.0f;  v_a[e+q+5]=-5.0f; // 2. vrchol
 v_a[e+q+6]= 5.0f; v_a[e+q+7]=10.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=0.0f; // 2. vrchol
 t_a[72+w+4]=1.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 TBN(e,q,w);
 q+=9; w+=6;

 v_a[e+q+0]=-5.0f; v_a[e+q+1]=0.0f; v_a[e+q+2]=-5.0f; // 1. vrchol
 v_a[e+q+3]= 5.0f; v_a[e+q+4]=10.0f; v_a[e+q+5]=-5.0f; // 2. vrchol
 v_a[e+q+6]=-5.0f; v_a[e+q+7]=10.0f; v_a[e+q+8]=-5.0f; // 3. vrchol
 c_a[e+q+0]=0.2f; c_a[e+q+1]=0.2f; c_a[e+q+2]=0.2f; // 1. vrchol
 c_a[e+q+3]=0.2f; c_a[e+q+4]=0.2f; c_a[e+q+5]=0.2f; // 2. vrchol
 c_a[e+q+6]=0.2f; c_a[e+q+7]=0.2f; c_a[e+q+8]=0.2f; // 3. vrchol
 t_a[72+w+0]=0.0f; t_a[72+w+1]=0.0f; // 1. vrchol
 t_a[72+w+2]=1.0f; t_a[72+w+3]=1.0f; // 2. vrchol
 t_a[72+w+4]=0.0f; t_a[72+w+5]=1.0f; // 3. vrchol
 TBN(e,q,w);
 q+=9; w+=6;

 objs.vrts_num=e/3+q/3; // kocka+gula+valec+ihlan+plochy
 printf("pocet trojuholnikov: %i\n",objs.vrts_num/3);

 // priprava objektov - kocka, gula, valec, ihlan, plochy
 objs.buffers_prepare();
 // nahratie udajov do zasobnikov
 objs.vertices->load(sizeof(v_a),v_a);
 objs.colors->load(sizeof(c_a),c_a);
 objs.tangents->load(sizeof(tan_a),tan_a);
 objs.bitangents->load(sizeof(bit_a),bit_a);
 objs.normals->load(sizeof(nor_a),nor_a);
 objs.texcoords->load(sizeof(t_a),t_a);

#ifdef OGL20
 // vrcholy
 glEnableVertexAttribArray(0); // povolenie poli atributov
 // farby
 glEnableVertexAttribArray(1);
 // tangenty
 glEnableVertexAttribArray(2);
 // bitangenty
 glEnableVertexAttribArray(3);
 // normaly
 glEnableVertexAttribArray(4);
 // texturovacie suradnice
 glEnableVertexAttribArray(8);
#else
 // vrcholy
 glEnableVertexAttribArrayARB(0);
 // farby
 glEnableVertexAttribArrayARB(1);
 // tangenty
 glEnableVertexAttribArrayARB(2);
 // bitangenty
 glEnableVertexAttribArrayARB(3);
 // normaly
 glEnableVertexAttribArrayARB(4);
 // texturovacie suradnice
 glEnableVertexAttribArrayARB(8);
#endif

 // nastavenie ukazovatelov na udaje objektov
 objects_bind();

 // INFO: bez shaderov
/*
 glEnableClientState(GL_VERTEX_ARRAY);
 glVertexPointer(3,GL_FLOAT,0,v_a);
 glEnableClientState(GL_COLOR_ARRAY);
 glColorPointer(3,GL_FLOAT,0,c_a);
*/
}

// nastavenie ukazovatelov na udaje objektov *******************************
void CInterface::objects_bind(void)
{
#ifdef OGL20
 // vrcholy
 objs.vertices->bind();                            // pripojenie buffra
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,0); // ukazovatel na udaje
 // farby
 objs.colors->bind();
 glVertexAttribPointer(1,3,GL_FLOAT,GL_FALSE,0,0);
 // tangenty
 objs.tangents->bind();
 glVertexAttribPointer(2,3,GL_FLOAT,GL_FALSE,0,0);
 // bitangenty
 objs.bitangents->bind();
 glVertexAttribPointer(3,3,GL_FLOAT,GL_FALSE,0,0);
 // normaly
 objs.normals->bind();
 glVertexAttribPointer(4,3,GL_FLOAT,GL_FALSE,0,0);
 // texturovacie suradnice
 objs.texcoords->bind();
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,0);
#else
 // vrcholy
 objs.vertices->bind();
 glVertexAttribPointerARB(0,3,GL_FLOAT,GL_FALSE,0,0);
 // farby
 objs.colors->bind();
 glVertexAttribPointerARB(1,3,GL_FLOAT,GL_FALSE,0,0);
 // tangenty
 objs.tangents->bind();
 glVertexAttribPointerARB(2,3,GL_FLOAT,GL_FALSE,0,0);
 // bitangenty
 objs.bitangents->bind();
 glVertexAttribPointerARB(3,3,GL_FLOAT,GL_FALSE,0,0);
 // normaly
 objs.normals->bind();
 glVertexAttribPointerARB(4,3,GL_FLOAT,GL_FALSE,0,0);
 // texturovacie suradnice
 objs.texcoords->bind();
 glVertexAttribPointerARB(8,2,GL_FLOAT,GL_FALSE,0,0);
#endif	
}

// odpojenie buffrov *******************************************************
void CInterface::objects_unbind(void)
{
 objs.vertices->unbind();
 objs.colors->unbind();
 objs.tangents->unbind();
 objs.bitangents->unbind();
 objs.normals->unbind();
 objs.texcoords->unbind();
}

// vykreslanie objektov - kocka, gula, valec, ihlan, plochy ****************
void CInterface::objects_draw(void)
{
 // vsetky objekty
 glDrawArrays(GL_TRIANGLES,0,objs.vrts_num);
 // iba plochy
 //glDrawArrays(GL_TRIANGLES,(4824-36)/3,12);
}

// mys je/nie je na tlacidle ***********************************************
bool CInterface::on_button(float X,float Y)
{
 // pre vsetky tlacidla
 loopi(BUTNUM) {
  // ak je mys na tlacidle
  if (but[i].on_button(X,Y)) { but_id=i; return(true); }
 }
 // mys nie je na tlacidle
 return(false);
}

// zmena udajov tlacidiel **************************************************
void CInterface::button_update(float X)
{
 // zmena udajov konkretneho tlacidla
 old_but_data=but[but_id].get_data();
 but[but_id].update(X/200.0f);
 new_but_data=but[but_id].get_data();
 // zmena parametrov osvetlenia podla zmenenych udajov tlacidiel
 switch (but_id) {
  case 0: lsrcparam.ambient.x=new_but_data; break;
  case 1: lsrcparam.ambient.y=new_but_data; break;
  case 2: lsrcparam.ambient.z=new_but_data; break;
  case 3: lsrcparam.ambient.w=new_but_data; break;
  case 4: lsrcparam.diffuse.x=new_but_data; break;
  case 5: lsrcparam.diffuse.y=new_but_data; break;
  case 6: lsrcparam.diffuse.z=new_but_data; break;
  case 7: lsrcparam.diffuse.w=new_but_data; break;
  case 8: lsrcparam.specular.x=new_but_data; break;
  case 9: lsrcparam.specular.y=new_but_data; break;
  case 10: lsrcparam.specular.z=new_but_data; break;
  case 11: lsrcparam.specular.w=new_but_data; break;
  case 12: lsrcparam.position.x=new_but_data; break;
  case 13: lsrcparam.position.y=new_but_data; break;
  case 14: lsrcparam.position.z=new_but_data; break;
  case 15: lsrcparam.position.w=new_but_data; break;
  case 16: lsrcparam.spotDirection.x=new_but_data; break;
  case 17: lsrcparam.spotDirection.y=new_but_data; break;
  case 18: lsrcparam.spotDirection.z=new_but_data; break;
  case 19: lsrcparam.spotExponent=new_but_data; break;
  case 20: lsrcparam.spotCutoff=new_but_data; break;
  case 21: lsrcparam.constantAttenuation=new_but_data; break;
  case 22: lsrcparam.linearAttenuation=new_but_data; break;
  case 23: lsrcparam.quadraticAttenuation=new_but_data; break;
  case 24: lmodelparam.ambient.x=new_but_data; break;
  case 25: lmodelparam.ambient.y=new_but_data; break;
  case 26: lmodelparam.ambient.z=new_but_data; break;
  case 27: lmodelparam.ambient.w=new_but_data; break;
  case 28: mparam.ambient.x=new_but_data; break;
  case 29: mparam.ambient.y=new_but_data; break;
  case 30: mparam.ambient.z=new_but_data; break;
  case 31: mparam.ambient.w=new_but_data; break;
  case 32: mparam.diffuse.x=new_but_data; break;
  case 33: mparam.diffuse.y=new_but_data; break;
  case 34: mparam.diffuse.z=new_but_data; break;
  case 35: mparam.diffuse.w=new_but_data; break;
  case 36: mparam.specular.x=new_but_data; break;
  case 37: mparam.specular.y=new_but_data; break;
  case 38: mparam.specular.z=new_but_data; break;
  case 39: mparam.specular.w=new_but_data; break;
  case 40: mparam.emission.x=new_but_data; break;
  case 41: mparam.emission.y=new_but_data; break;
  case 42: mparam.emission.z=new_but_data; break;
  case 43: mparam.emission.w=new_but_data; break;
  case 44: mparam.shininess=new_but_data; break;
  case 45: // zmena indexu textoveho udaju tlacidla = zmena pouziteho shadera
           but_text_id=(int) new_but_data;
           // zmena textoveho udaju tlacidla
           but[45].set_data(new_but_data,but_text[but_text_id]);
           if (but_text_id==5) mparam.diffuse.set(0.6f,0.3f,0.1f,1.0f);
           else mparam.diffuse.set(0.8f,0.8f,0.8f,1.0f);
           but[32].set_data(mparam.diffuse.x); but[33].set_data(mparam.diffuse.y);
           but[34].set_data(mparam.diffuse.z); but[35].set_data(mparam.diffuse.w);
           break;
  case 46: // reload shadera
           but_id=-1; // reload shadera sa vykona iba 1x
           switch (but_text_id) {
            case 0: break;
            case 1: break;
            case 2: break;
            case 3: break;
            case 4: break;
            case 5: wood_shader.reload();

                    wood_shader.on();
                    wood_shader.bindattriblocation(0,"in_pos"); // vrcholy
                    wood_shader.bindattriblocation(4,"in_nor"); // normaly
                    wood_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
                    wood_shader.uniform3F("lightWoodColor",0.6f,0.3f,0.1f);
                    wood_shader.uniform3F("darkWoodColor",0.4f,0.2f,0.07f);
                    wood_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
                    wood_shader.off();
                    break;
            case 6: nm_shader.reload();

                    nm_shader.on();
                    nm_shader.bindattriblocation(0,"in_pos"); // vrcholy
                    nm_shader.bindattriblocation(2,"in_tan"); // tangenty
                    nm_shader.bindattriblocation(3,"in_bit"); // bitangenty
                    nm_shader.bindattriblocation(4,"in_nor"); // normaly
                    nm_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
                    nm_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
                    nm_shader.uniformI("nm_tex_unit",1);      // normalmap texturovacia jednotka
                    nm_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
                    nm_shader.off();
                    break;
            case 7: px_shader.reload();

                    px_shader.on();
                    px_shader.bindattriblocation(0,"in_pos"); // vrcholy
                    px_shader.bindattriblocation(2,"in_tan"); // tangenty
                    px_shader.bindattriblocation(3,"in_bit"); // bitangenty
                    px_shader.bindattriblocation(4,"in_nor"); // normaly
                    px_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
                    px_shader.uniformI("base_tex_unit",0);    // zakladna texturovacia jednotka
                    px_shader.uniformI("hm_tex_unit",1);      // heightmap texturovacia jednotka
                    px_shader.uniformI("nm_tex_unit",2);      // normalmap texturovacia jednotka
                    px_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
                    px_shader.off();
                    break;
            case 8: water_shader.reload();

                    water_shader.on();
                    water_shader.bindattriblocation(0,"in_pos"); // vrcholy
                    water_shader.bindattriblocation(2,"in_tan"); // tangenty
                    water_shader.bindattriblocation(3,"in_bit"); // bitangenty
                    water_shader.bindattriblocation(4,"in_nor"); // normaly
                    water_shader.bindattriblocation(8,"in_uv");  // texturovacie suradnice
                    water_shader.uniformI("rfl_tex_unit",0);     // reflection texturovacia jednotka
                    water_shader.uniformI("rfr_tex_unit",1);     // refraction texturovacia jednotka
                    water_shader.uniformI("depth_tex_unit",2);   // depth texturovacia jednotka
                    water_shader.uniformI("nm_tex_unit",3);      // normalmap texturovacia jednotka
                    water_shader.uniformI("dudv_tex_unit",4);    // dudv texturovacia jednotka
                    water_shader.uniformmatrixFV("projection_mat",avatar.get_pr_mat()); // projekcna matica
                    water_shader.off();
                    break;
           }
           break;
  default: break;
 }

 // aktualizacia parametrov osvetlenia
 update_lighting_params();
}

// natocenie svetelneho zdroja *********************************************
void CInterface::light_rotate(GLfloat A,GLfloat B)
{
 lA+=-A; lB+=-B;

 lsrcparam.spotDirection.set(0.0f,0.0f,-1.0f);
 lsrcparam.spotDirection.rotate(lB,rightV);
 lsrcparam.spotDirection.rotate(lA,upV);

 but[16].set_data(lsrcparam.spotDirection.x); but[17].set_data(lsrcparam.spotDirection.y);
 but[18].set_data(lsrcparam.spotDirection.z);
}
